@@ -1,6 +1,6 @@
 /*      Orage - Calendar and alarm handler
  *
- * Copyright (c) 2005-2011 Juha Kautto  (juha at xfce.org)
+ * Copyright (c) 2005-2013 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2003-2005 Mickael Graf (korbinus at xfce.org)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -89,7 +89,7 @@ void program_log (const char *format, ...)
 void orage_message(gint level, const char *format, ...)
 {
     va_list args;
-    char *formatted;
+    char *formatted, time_stamp[10];
     struct tm *t = orage_localtime();
 
     if (level < g_log_level)
@@ -98,17 +98,17 @@ void orage_message(gint level, const char *format, ...)
     formatted = g_strdup_vprintf(format, args);
     va_end(args);
 
-    g_print("%02d:%02d:%02d ", t->tm_hour, t->tm_min, t->tm_sec);
+    g_sprintf(time_stamp, "%02d:%02d:%02d ", t->tm_hour, t->tm_min, t->tm_sec);
     if (level < 0)
-        g_debug("%s", formatted);
+        g_debug("%s%s", time_stamp, formatted);
     else if (level < 100) 
-        g_message("Orage **: %s", formatted);
+        g_message("Orage **: %s %s", time_stamp, formatted);
     else if (level < 200) 
-        g_warning("%s", formatted);
+        g_warning("%s%s", time_stamp, formatted);
     else if (level < 300) 
-        g_critical("%s", formatted);
+        g_critical("%s%s", time_stamp, formatted);
     else
-        g_error("Orage **: %s", formatted);
+        g_error("Orage **: %s%s", time_stamp, formatted);
     g_free(formatted);
 }
 
@@ -129,15 +129,18 @@ GtkWidget *orage_create_combo_box_with_content(char *text[], int size)
     return(combo_box);
 }
 
-gboolean orage_date_button_clicked(GtkWidget *button, GtkWidget *win)
+gboolean orage_date_button_clicked(GtkWidget *button, GtkWidget *selDate_dialog)
 {
-    GtkWidget *selDate_dialog;
+/*  GtkWidget *selDate_dialog; */
     GtkWidget *selDate_calendar;
     gint result;
     char *new_date=NULL, *cur_date;
     struct tm cur_t;
     gboolean changed, allocated=FALSE;
 
+    /*
+       For some unknown reason NLS does not work in this file, so this
+       has to be done in the main code:
     selDate_dialog = gtk_dialog_new_with_buttons(
             _("Pick the date"), GTK_WINDOW(win),
             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -146,6 +149,7 @@ gboolean orage_date_button_clicked(GtkWidget *button, GtkWidget *win)
             GTK_STOCK_OK,
             GTK_RESPONSE_ACCEPT,
             NULL);
+            */
 
     selDate_calendar = gtk_calendar_new();
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(selDate_dialog)->vbox)
@@ -228,14 +232,12 @@ gboolean orage_exec(const char *cmd, gboolean *cmd_active, GError **error)
 }
 
 GtkWidget *orage_toolbar_append_button(GtkWidget *toolbar
-    , const gchar *stock_id, GtkTooltips *tooltips, const char *tooltip_text
-    , gint pos)
+    , const gchar *stock_id, const char *tooltip_text, gint pos)
 {
     GtkWidget *button;
 
     button = (GtkWidget *)gtk_tool_button_new_from_stock(stock_id);
-    gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(button), tooltips
-            , (const gchar *) tooltip_text, NULL);
+    gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(button), tooltip_text);
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button), pos);
     return button;
 }
@@ -332,7 +334,7 @@ char *orage_replace_text(char *text, char *old, char *new)
 {
 #undef P_N
 #define P_N "orage_replace_text: "
-    /* these point to the original string and travel it until no more commands 
+    /* these point to the original string and travel it until no more olds 
      * are found:
      * cur points to the current head (after each old if any found)
      * cmd points to the start of next old in text */
@@ -368,6 +370,82 @@ char *orage_replace_text(char *text, char *old, char *new)
     return(beq);
 }
 
+/* This is helper function for orage_limit_text. It takes char pointer
+   to string start and length and adds that "line" to the beginning of 
+   old_result and returns the new_result.
+*/
+static char* add_line(char *old_result, char *start, int line_len
+        , int max_line_len)
+{
+    char *tmp;
+    char *line;          /* new line to add */
+    char *new_result;
+
+    if (line_len > max_line_len) {
+        tmp = g_strndup(start, max_line_len-3);
+        if (*(start+line_len-1) == '\n') { /* need to add line change */
+            line = g_strjoin("", tmp, "...\n", NULL);
+        }
+        else {
+            line = g_strjoin("", tmp, "...", NULL);
+        }
+        g_free(tmp);
+    }
+    else {
+        line = g_strndup(start, line_len);
+    }
+    /* note that old_result can be NULL */
+    new_result = g_strjoin("", line, old_result, NULL);
+    g_free(line);
+    g_free(old_result);
+    return(new_result);
+}
+
+/* cuts text to fit in a box defined with max_line_len and max_lines.
+   This is usefull for example in tooltips to avoid too broad and too may lines
+   to be visible.
+   You can always use this like 
+   str=orage_limit_text(str, max_line_len, max_lines);
+   but it may point to new place.
+*/
+char *orage_limit_text(char *text, int max_line_len, int max_lines)
+{
+#undef P_N
+#define P_N "orage_limit_text: "
+    /* these point to the original string and travel it until no more cuts 
+     * are needed:
+     * cur points to the current end of text character.
+     * eol is pointing to the end of current line */
+    char *cur, *eol;
+    char *result=NULL;   /* end result is collected to this */
+    int text_len=strlen(text);
+    int line_cnt=0;
+
+    if (text_len < 2) /* nothing to do */
+        return(text);
+    /* we start from the end and collect upto max_lines to a new buffer, but
+     * cut them to max_line_len before accepting */
+    for (cur = text+text_len-2, eol = text+text_len-1;
+         (cur > text) && (line_cnt < max_lines); cur--) {
+        if (*cur == '\n') {
+            /* line found, collect the line */
+            result = add_line(result, cur+1, eol-cur, max_line_len);
+            line_cnt++;
+            eol = cur;
+        }
+    }
+    /* Need to process first line also as it does not start with line end */
+    if ((cur == text) && (line_cnt < max_lines)) {
+        result = add_line(result, cur, eol-cur+1, max_line_len);
+    }
+    if (result) {
+        g_free(text);
+        return(result);
+    }
+    else 
+        return(text);
+}
+
 /* this will change <&Xnnn> type commands to numbers or text as defined.
  * Currently the only command is 
  * <&Ynnnn> which calculates years between current year and nnnn */
@@ -520,6 +598,18 @@ GtkWidget *orage_create_framebox_with_content(const gchar *title
     return(framebox);
 }
 
+GtkWidget *orage_create_custom_stock_button(const gchar *stock_id
+        , const gchar *label_text) {
+    GtkWidget *button;
+    GtkWidget *image;
+
+    image = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
+    button = gtk_button_new_with_label(label_text);
+    gtk_button_set_image((GtkButton*)button, image);
+    
+    return(button);
+}
+
 /*******************************************************
  * time convert and manipulation functions
  *******************************************************/
@@ -554,18 +644,18 @@ struct tm orage_i18_date_to_tm_date(const char *i18_date)
 
 char *orage_tm_time_to_i18_time(struct tm *tm_time)
 {
-    static char i18_time[40];
+    static char i18_time[128];
 
-    if (strftime(i18_time, 40, "%x %R", tm_time) == 0)
+    if (strftime(i18_time, 128, "%x %R", tm_time) == 0)
         g_error("Orage: orage_tm_time_to_i18_time too long string in strftime");
     return(i18_time);
 }
 
 char *orage_tm_date_to_i18_date(struct tm *tm_date)
 {
-    static char i18_date[32];
+    static char i18_date[128];
 
-    if (strftime(i18_date, 32, "%x", tm_date) == 0)
+    if (strftime(i18_date, 128, "%x", tm_date) == 0)
         g_error("Orage: orage_tm_date_to_i18_date too long string in strftime");
     return(i18_date);
 }
@@ -727,34 +817,9 @@ char *orage_localdate_i18(void)
 /* move one day forward or backward */
 void orage_move_day(struct tm *t, int day)
 {
-    gint monthdays[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
-
-    if (day < -1 || day > 1) {
-        g_warning("orage: orage_move_day wrong parameter %d", day);
-    }
-    t->tm_year += 1900;
-    if (((t->tm_year%4) == 0) 
-    && (((t->tm_year%100) != 0) || ((t->tm_year%400) == 0)))
-        ++monthdays[1]; /* leap year, february has 29 days */
-
     t->tm_mday += day; /* may be negative */
-    if (t->tm_mday == 0) { /*  we went to previous month */
-        if (--t->tm_mon == -1) { /* previous year */
-            --t->tm_year;
-            t->tm_mon = 11;
-        }
-        t->tm_mday = monthdays[t->tm_mon];
-    }
-    else if (t->tm_mday > (monthdays[t->tm_mon])) { /* next month */
-        if (++t->tm_mon == 12) {
-            ++t->tm_year;
-            t->tm_mon = 0;
-        }
-        t->tm_mday = 1;
-    }
-    t->tm_year -= 1900;
-    /* need to fill missing tm_wday and tm_yday, which are in use 
-     * in some locale's default date. For example in en_IN. mktime does it */
+    /* mktime adjusts t correctly. It also fills missing tm_wday and tm_yday, 
+     * which are in use in some locale's default date. For example in en_IN */
     if (mktime(t) == (time_t) -1) {
         g_warning("orage: orage_move_day mktime failed %d %d %d"
                 , t->tm_year, t->tm_mon, t->tm_mday);
@@ -817,7 +882,7 @@ gboolean orage_copy_file(gchar *source, gchar *target)
         ok = FALSE;
     }
     /* write file */
-    if (!g_file_set_contents(target, text, -1, &error)) {
+    if (ok && !g_file_set_contents(target, text, -1, &error)) {
         orage_message(150, "orage_copy_file: Could not write file (%s) error:%s"
                 , target, error->message);
         g_error_free(error);
@@ -1128,7 +1193,8 @@ gint orage_info_dialog(GtkWindow *parent
 }
 
 gint orage_warning_dialog(GtkWindow *parent
-        , char *primary_text, char *secondary_text)
+        , char *primary_text, char *secondary_text
+        , char *no_text, char *yes_text)
 {
     GtkWidget *dialog;
     gint result;
@@ -1136,11 +1202,13 @@ gint orage_warning_dialog(GtkWindow *parent
     dialog = gtk_message_dialog_new(parent
             , GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT
             , GTK_MESSAGE_WARNING
-            , GTK_BUTTONS_YES_NO
+            , GTK_BUTTONS_NONE
             , "%s", primary_text);
     if (secondary_text) 
         gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog)
                 , "%s", secondary_text);
+    gtk_dialog_add_buttons(GTK_DIALOG(dialog)
+            , no_text, GTK_RESPONSE_NO, yes_text, GTK_RESPONSE_YES, NULL);
     result = gtk_dialog_run(GTK_DIALOG(dialog));
     gtk_widget_destroy(dialog);
     return(result);
